@@ -1,8 +1,5 @@
 import re
 import sys
-from six.moves import urllib
-if sys.version < '2.4':
-    from sets import ImmutableSet as frozenset
 
 import six
 from six.moves.urllib import parse as urlparse
@@ -38,9 +35,9 @@ class Route(object):
             >>> newroute = Route(None, 'date/:year/:month/:day',
             ...     controller="blog", action="view")
             >>> newroute = Route(None, 'archives/:page', controller="blog",
-            ...     action="by_page", requirements = { 'page':'\d{1,2}' })
+            ...     action="by_page", requirements = { 'page':'\\d{1,2}' })
             >>> newroute.reqs
-            {'page': '\\\d{1,2}'}
+            {'page': '\\\\d{1,2}'}
 
         .. Note::
             Route is generally not called directly, a Mapper instance
@@ -149,13 +146,22 @@ class Route(object):
         """Utility function to walk the route, and pull out the valid
         dynamic/wildcard keys."""
         collecting = False
+        escaping = False
         current = ''
         done_on = ''
         var_type = ''
         just_started = False
         routelist = []
         for char in routepath:
-            if char in [':', '*', '{'] and not collecting and not self.static \
+            if escaping:
+                if char in ['\\', ':', '*', '{', '}']:
+                    current += char
+                else:
+                    current += '\\' + char
+                escaping = False
+            elif char == '\\':
+                escaping = True
+            elif char in [':', '*', '{'] and not collecting and not self.static \
                or char in ['{'] and not collecting:
                 just_started = True
                 collecting = True
@@ -328,7 +334,7 @@ class Route(object):
                 else:
                     regpart = '(?:%s)' % partmatch
                 if part['type'] == '.':
-                    regparts.append('(?:\.%s)??' % regpart)
+                    regparts.append(r'(?:\.%s)??' % regpart)
                 else:
                     regparts.append(regpart)
             else:
@@ -358,7 +364,6 @@ class Route(object):
             self.prior = part
             (rest, noreqs, allblank) = self.buildnextreg(path[1:], clist,
                                                          include_names)
-
         if isinstance(part, dict) and part['type'] in (':', '.'):
             var = part['name']
             typ = part['type']
@@ -371,7 +376,7 @@ class Route(object):
                 else:
                     partreg = '(?:%s)' % self.reqs[var]
                 if typ == '.':
-                    partreg = '(?:\.%s)??' % partreg
+                    partreg = r'(?:\.%s)??' % partreg
             elif var == 'controller':
                 if include_names:
                     partreg = '(?P<%s>%s)' % (var, '|'.join(map(re.escape,
@@ -394,7 +399,7 @@ class Route(object):
                     else:
                         partreg = '(?:[^%s]+?)' % exclude_chars
                     if typ == '.':
-                        partreg = '(?:\.%s)??' % partreg
+                        partreg = r'(?:\.%s)??' % partreg
                 else:
                     end = ''.join(self.done_chars)
                     rem = rest
@@ -496,7 +501,13 @@ class Route(object):
                 reg += ')?'
             else:
                 allblank = False
-                reg = re.escape(part) + rest
+                # Starting in Python 3.7, the / is no longer escaped, however quite a bit of
+                # route generation code relies on it being escaped. This forces the escape in
+                # Python 3.7+ so that the remainder of the code functions as intended.
+                if part == '/':
+                    reg = r'\/' + rest
+                else:
+                    reg = re.escape(part) + rest
 
         # We have a normal string here, this is a req, and it prevents us from
         # being all blank
@@ -534,7 +545,7 @@ class Route(object):
 
         if sub_domains and environ and 'HTTP_HOST' in environ:
             host = environ['HTTP_HOST'].split(':')[0]
-            sub_match = re.compile('^(.+?)\.%s$' % domain_match)
+            sub_match = re.compile(r'^(.+?)\.%s$' % domain_match)
             subdomain = re.sub(sub_match, r'\1', host)
             if subdomain not in sub_domains_ignore and host != subdomain:
                 sub_domain = subdomain
